package com.lowes.tcl.app.pemw.cpo.persistance.dbaction;

import java.util.ArrayList;
import java.util.List;

import com.lowes.so.lkn.core.LKNException;
import com.lowes.so.lkn.log.LoggingConstants;
import com.lowes.tcl.api.pem.PemApiConstants;
import com.lowes.tcl.api.pem.PemApiServices;
import com.lowes.tcl.api.pem.PemDBAction;
import com.lowes.tcl.api.pem.cgiserver.PemApiGenericSQLHandler;
import com.lowes.tcl.api.pem.cgiserver.PemApiServicesImpl;
import com.lowes.tcl.api.pem.model.InstallerInvoice;
import com.lowes.tcl.api.pem.model.Order;
import com.lowes.tcl.api.pem.model.Vbu;
import com.lowes.tcl.app.pemw.cpo.model.CodesAndTypesDao;
import com.lowes.tcl.app.pemw.cpo.model.SaleTypeCode;

/**
 * Selects a count of projects which where sold in the given period of time.
 * <p>
 * This action counts projects which acquired SOLD status in the given period of time.
 * <p>
 * Created: Fri Oct 03 14:23:45 EDT 2008
 *
 * @author <a href="mailto:vc.julian.nune@lowes.com">Julian F. Nunez</a>
 */
public class PemGetExceptionPaymentsOrdersAction extends PemDBAction implements LoggingConstants, PemApiConstants {

    private static final String SQL = initializeSQL();

    private CodesAndTypesDao typesDao;
    
    private List result = new ArrayList();

	/**
     * Constructor for class SelectSoldProjectCountByRangeAction
     * 
     * @param aDBConnection
     *            A connection to the database.
     */
    public PemGetExceptionPaymentsOrdersAction(int storeNumber, CodesAndTypesDao typesDao) {
    	super(storeNumber);
    	this.typesDao = typesDao;
    }

    
    /**
     * Handles the workflow of one or more logically related SQL calls.
     * <p>
     * This method performs logging of the parameters and SQL, calls
     * validateParameters, creates the SQL statement usually as a
     * PreparedStatement, executes the statement and stores the results
     * available through getResult. Any ResultSet, Statement, and
     * PreparedStatement objects that are instantiated are explicitly closed.
     * Transaction logic such as Commits and Rollbacks are not included and are
     * typically handled by the caller of this method.
     * 
     * @throws LKNException
     *             If the entire workflow did not complete successfully
     *             typically due to database or parameter related issues.
     * @see com.lowes.so.lkn.db.DBAction#execute()
     */
    public void execute() throws LKNException {
		
    	PemApiServices pemApiServices = new PemApiServicesImpl(storeNumber);
        
		log(DEBUG, SQL);
		
		try {
			
			PemApiGenericSQLHandler.ResultReader reader = pemApiServices.runQuery(SQL);
			
			while(reader.next()) {
				
				InstallerInvoice invoice = new InstallerInvoice();

	        	invoice.setLocationNbr(storeNumber);
	        	invoice.setPurchaseOrdNbr(reader.getInt(1));
	        	
	        	Order order = new Order();
	        	invoice.setOrder(order);

	        	order.setLocationNbr(storeNumber);
	        	order.setPurchaseOrdNbr(reader.getInt(1));

	        	Vbu installer = new Vbu();
	        	installer.setVbuName(reader.getString(2));
	        	installer.setVbuNumber(reader.getInt(3));
	        	order.setInstaller(installer);	     	        	
	        	
	        	invoice.setAdditionDate(reader.getDate(4));
	        	invoice.setInvoiceNbr(reader.getString(5));
	        	invoice.setInvoiceTotalAmount(reader.getFloat(6));
	        	
	        	SaleTypeCode type = typesDao.getSaleTypeCodeById(reader.getInt(7));
	        	order.setSaleTypeCode(type.getShortDescriptionText());
	        	
	        	result.add(invoice);
			}
		}
		catch (Exception e) {
			throw new LKNException(e);
		}
		finally {
			//close(resultSet);
		}        
    }

    /**
     * Returns the result of the database action as an Object. 
     * <p>
     * It is the caller's responsibility to know how to cast the object to its 
     * appropriate type based on this implementation.
     * 
     * @return A List of CommunicationEventType objects that matched the criteria.
     */
    public Object getResult() {
        return result;
    }

    /**
     * Generates the SQL that will be executed.  
     * <p>
     * The static signature with no parameters is required to take advantage of 
     * LKN's SQL intropection tools. 
     * 
     * @return the SQL query to execute
     */
    public static String initializeSQL()
    {
        StringBuffer sb = new StringBuffer(); 

        sb.append("SELECT "); 
        sb.append("   A.E100_PO_NBR, "); 
        sb.append("   B.VND_NME, "); 
        sb.append("   A.P028_VND_NBR, "); 
        sb.append("   convert(char(10), A.EMP_CNF_DM, 112), "); 
        sb.append("   A.E559_IST_IVO_NBR, "); 
        sb.append("   A.IST_IVO_AMT, "); 
        sb.append("   C.SAL_TYP_CD "); 
        sb.append("FROM E559 A, "); 
        sb.append("     P082 B, "); 
        sb.append("     E161 C, "); 
        sb.append("     E100A D "); 
        sb.append("WHERE A.TNS_STS_CD = 2 "); 
        sb.append("   AND A.E100_PO_NBR = D.E100_PO_NBR "); 
        sb.append("   AND D.E161_PRJ_NBR = C.E161_PRJ_NBR "); 
        sb.append("   AND A.P028_VND_NBR = B.P028_VND_NBR "); 
        sb.append("ORDER BY A.P028_VND_NBR ");
        sb.append("FOR READ ONLY ");
        
        return sb.toString();
    }
}