package com.lowes.tcl.app.pemw.cpo.persistance.dbaction;

import java.util.ArrayList;
import java.util.List;

import com.lowes.so.lkn.core.LKNException;
import com.lowes.so.lkn.log.LoggingConstants;
import com.lowes.tcl.api.pem.PemApiConstants;
import com.lowes.tcl.api.pem.PemApiServices;
import com.lowes.tcl.api.pem.PemDBAction;
import com.lowes.tcl.api.pem.cgiserver.PemApiGenericSQLHandler;
import com.lowes.tcl.api.pem.cgiserver.PemApiServicesImpl;
import com.lowes.tcl.api.pem.model.Customer;
import com.lowes.tcl.api.pem.model.Order;
import com.lowes.tcl.api.pem.model.Status;
import com.lowes.tcl.app.pemw.cpo.model.CodesAndTypesDao;
import com.lowes.tcl.app.pemw.cpo.model.SaleTypeCode;

/**
 * Selects INS-X Action Item orders from the store.
 * <p>
 * This action counts projects which acquired SOLD status in the given period of time.
 * <p>
 * Created: Fri Oct 03 14:23:45 EDT 2008
 *
 * @author <a href="mailto:vc.julian.nune@lowes.com">Julian F. Nunez</a>
 */
public class PemGetInsxActionOrdersAction extends PemDBAction implements LoggingConstants, PemApiConstants {

	private String SQL;

    private CodesAndTypesDao typesDao;
    
    private boolean autofaxEnabled;
    
    private List result = new ArrayList();

	/**
     * Constructor for class SelectSoldProjectCountByRangeAction
     * 
     * @param aDBConnection
     *            A connection to the database.
     */
    public PemGetInsxActionOrdersAction(int storeNumber, CodesAndTypesDao typesDao, boolean autofaxEnabled) {
    	super(storeNumber);
    	this.typesDao = typesDao;
    	this.autofaxEnabled = autofaxEnabled;
    }

    
    /**
     * Handles the workflow of one or more logically related SQL calls.
     * <p>
     * This method performs logging of the parameters and SQL, calls
     * validateParameters, creates the SQL statement usually as a
     * PreparedStatement, executes the statement and stores the results
     * available through getResult. Any ResultSet, Statement, and
     * PreparedStatement objects that are instantiated are explicitly closed.
     * Transaction logic such as Commits and Rollbacks are not included and are
     * typically handled by the caller of this method.
     * 
     * @throws LKNException
     *             If the entire workflow did not complete successfully
     *             typically due to database or parameter related issues.
     * @see com.lowes.so.lkn.db.DBAction#execute()
     */
    public void execute() throws LKNException {
		
    	PemApiServices pemApiServices = new PemApiServicesImpl(storeNumber);
        
    	SQL = initializeSQL();
    	SQL = SQL.replaceFirst("\\?", autofaxEnabled ? "0" : "1");   
    	
		log(DEBUG, SQL);
		
		try {
			
			PemApiGenericSQLHandler.ResultReader reader = pemApiServices.runQuery(SQL);
			
			while(reader.next()) {
				
	        	Order order = new Order();

	        	Customer customer = new Customer();
	        	customer.setFirstName(reader.getString(1));
	        	customer.setLastName(reader.getString(2));
	        	order.setCustomer(customer);
	        	
	        	order.setLocationNbr(storeNumber);
	        	order.setProjectNbr(reader.getInt(3));
	        	order.setPurchaseOrderDesc(reader.getString(4));
	        	order.setAdditionDate(reader.getDate(5));
	        	
	        	SaleTypeCode type = typesDao.getSaleTypeCodeById(reader.getInt(6));
	        	order.setSaleTypeCode(type.getShortDescriptionText());
	        	
	        	Status status = new Status();
	        	status.setDescription(reader.getString(7));
	        	order.setStatus(status);
	        	
	        	order.setSuspended(reader.getBoolean(8));
	        	if(order.isSuspended()) {
	        		status.setDescription("SUSPEND");
	        	}
	        	
	        	result.add(order);
			}
		}
		catch (Exception e) {
			throw new LKNException(e);
		}
		finally {
			//close(resultSet);
		}        
    }

    /**
     * Returns the result of the database action as an Object. 
     * <p>
     * It is the caller's responsibility to know how to cast the object to its 
     * appropriate type based on this implementation.
     * 
     * @return A List of CommunicationEventType objects that matched the criteria.
     */
    public Object getResult() {
        return result;
    }

    /**
     * Generates the SQL that will be executed.  
     * <p>
     * The static signature with no parameters is required to take advantage of 
     * LKN's SQL intropection tools. 
     * 
     * @return the SQL query to execute
     */
    public static String initializeSQL()
    {
        StringBuffer sb = new StringBuffer(); 

        sb.append("SELECT DISTINCT "); 
        sb.append("  C.CUS_FIR_NME, "); 
        sb.append("  C.CUS_LST_NME, "); 
        sb.append("  A.E161_PRJ_NBR, "); 
        sb.append("  B.PRJ_DES_TXT, "); 
        sb.append("  CONVERT(char(8), F.ADD_DM, 112), "); 
        sb.append("  B.SAL_TYP_CD, "); 
        sb.append("  D.DES_TXT, "); 
        sb.append("  E.SPN_IDC "); 
        sb.append("FROM   E543   A, "); 
        sb.append("   E161   B, "); 
        sb.append("   P008   C, "); 
        sb.append("   T1477  D, "); 
        sb.append("   E100A  E, "); 
        sb.append("   T1596  F, "); 
        sb.append("   E549   G "); 
        sb.append("WHERE  A.E161_PRJ_NBR      = B.E161_PRJ_NBR "); 
        sb.append("AND    A.FUL_RTR_IDC      != 'Y' "); 
        sb.append("AND    B.SAL_TYP_CD       IN (1, 3) "); 
        sb.append("AND    A.P008_CUS_NBR      = C.P008_CUS_NBR "); 
        sb.append("AND    A.E161_PRJ_NBR      = E.E161_PRJ_NBR "); 
        sb.append("AND    E.ORD_IDC           = 'L' "); 
        sb.append("AND    E.SPN_IDC          != 'Y' "); 
        sb.append("AND    A.E161_PRJ_NBR      = F.T1420_SOS_PRJ_NBR "); 
        sb.append("AND    E.E100_PO_NBR       = F.T1420_SOS_PO_NBR "); 
        sb.append("AND    A.T1477_ORD_STS_CD  = F.T1477_ORD_STS_CD "); 
        sb.append("AND    A.T1477_ORD_STS_CD  = D.T1477_ORD_STS_CD "); 
        sb.append("AND    A.E549_IST_TYP_CD   = G.E549_IST_TYP_CD "); 
        sb.append("AND    A.T1477_ORD_STS_CD > 1 "); 
        sb.append("AND    F.ADD_DM = (SELECT MAX(ADD_DM) "); 
        sb.append("          FROM   T1596 "); 
        sb.append("          WHERE  T1420_SOS_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("          AND    T1477_ORD_STS_CD = A.T1477_ORD_STS_CD "); 
        sb.append("          AND    T1420_SOS_PO_NBR = E.E100_PO_NBR) "); 
        sb.append("AND    D.SMY_STS_TXT != 'COMPLETED' "); 
        sb.append("AND    ( "); 
        sb.append("         (A.FWP_DM >  '01/01/1901 12:00AM' AND "); 
        sb.append("          A.FWP_DM <= GETDATE() "); 
        sb.append("         ) OR "); 
        sb.append("         (A.IST_PKU_DM >  '01/01/1901 12:00AM' AND "); 
        sb.append("          A.IST_PKU_DM <= GETDATE() AND "); 
        sb.append("          A.FWP_DM      = '01/01/1901 12:00AM' "); 
        sb.append("         ) OR "); 
        sb.append("         (A.FWP_DM <= '01/01/1901 12:00AM' AND "); 
        sb.append("            ( "); 
        sb.append("               (D.SMY_STS_TXT = 'SOLD' AND "); 
        sb.append("                  ( "); 
        sb.append("                     ( "); 
        sb.append("                        (A.T3722_PRJ_FAX_CD IN (2, 3, 6, 7) OR ? = 1) AND "); 
        sb.append("                         NOT EXISTS (SELECT 1 "); 
        sb.append("                                     FROM   T1596 H "); 
        sb.append("                                     WHERE  H.T1420_SOS_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("                                     AND    H.T1420_SOS_PO_NBR = E.E100_PO_NBR "); 
        sb.append("                                     AND    H.T1477_ORD_STS_CD  = 6) "); 
        sb.append("                     ) OR "); 
        sb.append("                     ( "); 
        sb.append("                        (NOT EXISTS (SELECT 1 "); 
        sb.append("                                     FROM   T1596 H "); 
        sb.append("                                     WHERE  H.T1420_SOS_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("                                     AND    H.T1420_SOS_PO_NBR = E.E100_PO_NBR "); 
        sb.append("                                     AND    H.T1477_ORD_STS_CD = 7)) "); 
        sb.append("                     ) "); 
        sb.append("                  ) "); 
        sb.append("               ) OR "); 
        sb.append("               (D.SMY_STS_TXT = 'RECEIVED' "); 
        sb.append("               ) OR "); 
        sb.append("               (D.SMY_STS_TXT = 'NOTIFIED' AND "); 
        sb.append("                DATEDIFF(HOUR, F.ADD_DM, GETDATE()) >= 48 "); 
        sb.append("               ) OR "); 
        sb.append("               (D.SMY_STS_TXT = 'SCHEDULED' AND "); 
        sb.append("                  ( "); 
        sb.append("                     ( "); 
        sb.append("                         NOT EXISTS (SELECT 1 "); 
        sb.append("                                     FROM   T1596 I "); 
        sb.append("                                     WHERE  I.T1420_SOS_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("                                     AND    I.T1420_SOS_PO_NBR = E.E100_PO_NBR "); 
        sb.append("                                     AND    I.T1477_ORD_STS_CD = 101) "); 
        sb.append("                     ) OR "); 
        sb.append("                     ( "); 
        sb.append("                        (DATEDIFF(DAY, A.IST_END_DT, GETDATE()) > 0) AND "); 
        sb.append("                         NOT EXISTS (SELECT 1 "); 
        sb.append("                                     FROM   T1596 J "); 
        sb.append("                                     WHERE J.T1420_SOS_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("                                     AND J.T1420_SOS_PO_NBR = E.E100_PO_NBR "); 
        sb.append("                                     AND J.T1477_ORD_STS_CD = 102) "); 
        sb.append("                     ) "); 
        sb.append("                  ) "); 
        sb.append("               ) "); 
        sb.append("            ) "); 
        sb.append("         ) "); 
        sb.append("      ) "); 
        sb.append("FOR READ ONLY ");
        
        return sb.toString();
    }
}