package com.lowes.tcl.app.pemw.cpo.persistance.dbaction;

import java.util.ArrayList;
import java.util.List;

import com.lowes.so.lkn.core.LKNException;
import com.lowes.so.lkn.log.LoggingConstants;
import com.lowes.tcl.api.pem.PemApiConstants;
import com.lowes.tcl.api.pem.PemApiServices;
import com.lowes.tcl.api.pem.PemDBAction;
import com.lowes.tcl.api.pem.cgiserver.PemApiGenericSQLHandler;
import com.lowes.tcl.api.pem.cgiserver.PemApiServicesImpl;
import com.lowes.tcl.api.pem.model.Customer;
import com.lowes.tcl.api.pem.model.Order;
import com.lowes.tcl.api.pem.model.Status;
import com.lowes.tcl.app.pemw.cpo.model.CodesAndTypesDao;
import com.lowes.tcl.app.pemw.cpo.model.SaleTypeCode;

/**
 * Selects a count of projects which where sold in the given period of time.
 * <p>
 * This action counts projects which acquired SOLD status in the given period of time.
 * <p>
 * Created: Fri Oct 03 14:23:45 EDT 2008
 *
 * @author <a href="mailto:vc.julian.nune@lowes.com">Julian F. Nunez</a>
 */
public class PemGetScheduledNotInstalledOrdersAction extends PemDBAction implements LoggingConstants, PemApiConstants {

    private static final String SQL = initializeSQL();

    private CodesAndTypesDao typesDao;
    
    private List result = new ArrayList();

	/**
     * Constructor for class SelectSoldProjectCountByRangeAction
     * 
     * @param aDBConnection
     *            A connection to the database.
     */
    public PemGetScheduledNotInstalledOrdersAction(int storeNumber, CodesAndTypesDao typesDao) {
    	super(storeNumber);
    	this.typesDao = typesDao;
    }

    
    /**
     * Handles the workflow of one or more logically related SQL calls.
     * <p>
     * This method performs logging of the parameters and SQL, calls
     * validateParameters, creates the SQL statement usually as a
     * PreparedStatement, executes the statement and stores the results
     * available through getResult. Any ResultSet, Statement, and
     * PreparedStatement objects that are instantiated are explicitly closed.
     * Transaction logic such as Commits and Rollbacks are not included and are
     * typically handled by the caller of this method.
     * 
     * @throws LKNException
     *             If the entire workflow did not complete successfully
     *             typically due to database or parameter related issues.
     * @see com.lowes.so.lkn.db.DBAction#execute()
     */
    public void execute() throws LKNException {
		
    	PemApiServices pemApiServices = new PemApiServicesImpl(storeNumber);
        
		log(DEBUG, SQL);
		
		try {
			
			PemApiGenericSQLHandler.ResultReader reader = pemApiServices.runQuery(SQL);
			
			while(reader.next()) {
				
	        	Order order = new Order();

	        	Customer customer = new Customer();
	        	customer.setFirstName(reader.getString(1));
	        	customer.setLastName(reader.getString(2));
	        	order.setCustomer(customer);
	        	
	        	order.setLocationNbr(storeNumber);
	        	order.setProjectNbr(reader.getInt(3));
	        	order.setPurchaseOrderDesc(reader.getString(4));
	        	order.setAdditionDate(reader.getDateTime(5, 6));
	        	
	        	SaleTypeCode type = typesDao.getSaleTypeCodeById(reader.getInt(7));
	        	order.setSaleTypeCode(type.getShortDescriptionText());
	        	
	        	Status status = new Status();
	        	status.setDescription(reader.getString(8));
	        	order.setStatus(status);

	        	order.setSuspended(reader.getBoolean(10));
	        	if(order.isSuspended()) {
	        		status.setDescription("SUSPEND");
	        	}
	        	
	        	result.add(order);
			}
		}
		catch (Exception e) {
			throw new LKNException(e);
		}
		finally {
			//close(resultSet);
		}        
    }

    /**
     * Returns the result of the database action as an Object. 
     * <p>
     * It is the caller's responsibility to know how to cast the object to its 
     * appropriate type based on this implementation.
     * 
     * @return A List of CommunicationEventType objects that matched the criteria.
     */
    public Object getResult() {
        return result;
    }

    /**
     * Generates the SQL that will be executed.  
     * <p>
     * The static signature with no parameters is required to take advantage of 
     * LKN's SQL intropection tools. 
     * 
     * @return the SQL query to execute
     */
    public static String initializeSQL()
    {
        StringBuffer sb = new StringBuffer(); 

        //QUERY ADAPTED FROM omzl100DB.c, getOrdersByScheduledNotInstalled()
        sb.append("SELECT DISTINCT "); 
        sb.append("    C.CUS_FIR_NME, "); 
        sb.append("    C.CUS_LST_NME, "); 
        sb.append("    A.E161_PRJ_NBR, "); 
        sb.append("    str_replace(B.PRJ_DES_TXT,';',''), "); 
        sb.append("    CONVERT(char(10), F.ADD_DM, 112), "); 
        sb.append("    CONVERT(char(10), F.ADD_DM, 108), "); 
        sb.append("    B.SAL_TYP_CD, "); 
        sb.append("    D.DES_TXT, "); 
        sb.append("    G.SHT_DES_TXT, "); 
        sb.append("    E.SPN_IDC "); 
        sb.append("FROM  E543   A, "); 
        sb.append("      E161   B, "); 
        sb.append("      P008   C, "); 
        sb.append("      T1477  D, "); 
        sb.append("      E100A  E, "); 
        sb.append("      T1596  F, "); 
        sb.append("      E549   G "); 
        sb.append("WHERE  A.T1477_ORD_STS_CD  = D.T1477_ORD_STS_CD "); 
        sb.append("AND    A.FUL_RTR_IDC      != 'Y' "); 
        sb.append("AND    D.SMY_STS_TXT       = 'SCHEDULED' "); 
        sb.append("AND    B.SAL_TYP_CD        = 1 "); 
        sb.append("AND    A.IST_BGN_DT        > '01/01/2000' "); 
        sb.append("AND    A.IST_END_DT        > '01/01/2000' "); 
        sb.append("AND    A.E161_PRJ_NBR      = B.E161_PRJ_NBR "); 
        sb.append("AND    A.P008_CUS_NBR      = C.P008_CUS_NBR "); 
        sb.append("AND    A.E161_PRJ_NBR      = E.E161_PRJ_NBR "); 
        sb.append("AND    E.ORD_IDC           = 'L' "); 
        sb.append("AND    F.T1420_SOS_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("AND    F.T1420_SOS_PO_NBR  = E.E100_PO_NBR "); 
        sb.append("AND    F.T1477_ORD_STS_CD  = A.T1477_ORD_STS_CD "); 
        sb.append("AND    F.ADD_DM            = (SELECT MAX(ADD_DM) "); 
        sb.append("                             FROM   T1596 "); 
        sb.append("                             WHERE  T1420_SOS_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("                             AND    T1420_SOS_PO_NBR  = E.E100_PO_NBR "); 
        sb.append("                             AND    T1477_ORD_STS_CD  = A.T1477_ORD_STS_CD) "); 
        sb.append("AND    A.E549_IST_TYP_CD   = G.E549_IST_TYP_CD "); 
        sb.append("AND    EXISTS (SELECT 1 "); 
        sb.append("              FROM   T1596 H "); 
        sb.append("              WHERE  H.T1420_SOS_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("              AND    H.T1420_SOS_PO_NBR  = E.E100_PO_NBR "); 
        sb.append("              AND    H.T1477_ORD_STS_CD  = 100) "); 
        sb.append("ORDER BY F.ADD_DM "); 
        sb.append("FOR READ ONLY ");
        
        return sb.toString();
    }
}