package com.lowes.tcl.app.pemw.cpo.persistance.dbaction;

import java.util.ArrayList;
import java.util.List;

import com.lowes.so.lkn.core.LKNException;
import com.lowes.so.lkn.log.LoggingConstants;
import com.lowes.tcl.api.pem.PemApiConstants;
import com.lowes.tcl.api.pem.PemApiServices;
import com.lowes.tcl.api.pem.PemDBAction;
import com.lowes.tcl.api.pem.cgiserver.PemApiGenericSQLHandler;
import com.lowes.tcl.api.pem.cgiserver.PemApiServicesImpl;
import com.lowes.tcl.api.pem.model.InstallerInvoice;
import com.lowes.tcl.api.pem.model.Order;
import com.lowes.tcl.api.pem.model.Vbu;
import com.lowes.tcl.app.pemw.cpo.model.CodesAndTypesDao;
import com.lowes.tcl.app.pemw.cpo.model.SaleTypeCode;

/**
 * Selects a count of projects which where sold in the given period of time.
 * <p>
 * This action counts projects which acquired SOLD status in the given period of time.
 * <p>
 * Created: Fri Oct 03 14:23:45 EDT 2008
 *
 * @author <a href="mailto:vc.julian.nune@lowes.com">Julian F. Nunez</a>
 */
public class PemGetSubmittedPaymentsOrdersAction extends PemDBAction implements LoggingConstants, PemApiConstants {

    private static final String SQL = initializeSQL();

    private CodesAndTypesDao typesDao;
    
    private List result = new ArrayList();

	/**
     * Constructor for class SelectSoldProjectCountByRangeAction
     * 
     * @param aDBConnection
     *            A connection to the database.
     */
    public PemGetSubmittedPaymentsOrdersAction(int storeNumber, CodesAndTypesDao typesDao) {
    	super(storeNumber);
    	this.typesDao = typesDao;
    }

    
    /**
     * Handles the workflow of one or more logically related SQL calls.
     * <p>
     * This method performs logging of the parameters and SQL, calls
     * validateParameters, creates the SQL statement usually as a
     * PreparedStatement, executes the statement and stores the results
     * available through getResult. Any ResultSet, Statement, and
     * PreparedStatement objects that are instantiated are explicitly closed.
     * Transaction logic such as Commits and Rollbacks are not included and are
     * typically handled by the caller of this method.
     * 
     * @throws LKNException
     *             If the entire workflow did not complete successfully
     *             typically due to database or parameter related issues.
     * @see com.lowes.so.lkn.db.DBAction#execute()
     */
    public void execute() throws LKNException {
		
    	PemApiServices pemApiServices = new PemApiServicesImpl(storeNumber);
        
		log(DEBUG, SQL);
		
		try {
			
			PemApiGenericSQLHandler.ResultReader reader = pemApiServices.runQuery(SQL);
			
			while(reader.next()) {
				
				InstallerInvoice invoice = new InstallerInvoice();

	        	invoice.setLocationNbr(storeNumber);
	        	invoice.setProjectNbr(reader.getInt(1));
	        	invoice.setPurchaseOrdNbr(reader.getInt(2));
	        	
	        	Order order = new Order();
	        	invoice.setOrder(order);

	        	order.setLocationNbr(storeNumber);
	        	order.setProjectNbr(reader.getInt(1));
	        	order.setPurchaseOrdNbr(reader.getInt(2));

	        	Vbu installer = new Vbu();
	        	installer.setVbuName(reader.getString(3));
	        	installer.setVbuNumber(reader.getInt(4));
	        	order.setInstaller(installer);	     	        	
	        	
	        	invoice.setAdditionDate(reader.getDate(5));
	        	invoice.setInvoiceNbr(reader.getString(6));
	        	
	        	SaleTypeCode type = typesDao.getSaleTypeCodeById(reader.getInt(7));
	        	order.setSaleTypeCode(type.getShortDescriptionText());
	        	
	        	result.add(invoice);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			throw new LKNException(e);
		}
		finally {
			//close(resultSet);
		}        
    }

    /**
     * Returns the result of the database action as an Object. 
     * <p>
     * It is the caller's responsibility to know how to cast the object to its 
     * appropriate type based on this implementation.
     * 
     * @return A List of CommunicationEventType objects that matched the criteria.
     */
    public Object getResult() {
        return result;
    }

    /**
     * Generates the SQL that will be executed.  
     * <p>
     * The static signature with no parameters is required to take advantage of 
     * LKN's SQL intropection tools. 
     * 
     * @return the SQL query to execute
     */
    public static String initializeSQL()
    {
        StringBuffer sb = new StringBuffer(); 

        sb.append("SELECT "); 
        sb.append("    A.E161_PRJ_NBR, "); 
        sb.append("    A.E100A_LCT_PO_NBR, "); 
        sb.append("    B.VND_NME, "); 
        sb.append("    C.P028_VND_NBR, "); 
        sb.append("    convert(char(10), A.PAY_CNF_DM, 112), "); 
        sb.append("    A.IST_IVO_NBR, "); 
        sb.append("    D.SAL_TYP_CD "); 
        sb.append("FROM E474 A, "); 
        sb.append("    P082 B, "); 
        sb.append("    E543 C, "); 
        sb.append("    E161 D "); 
        sb.append("WHERE A.E161_PRJ_NBR = C.E161_PRJ_NBR "); 
        sb.append("    AND A.T546_IST_PAY_CD = 2 "); 
        sb.append("    AND C.P028_VND_NBR = B.P028_VND_NBR "); 
        sb.append("    AND A.E161_PRJ_NBR = D.E161_PRJ_NBR "); 
        sb.append("    AND 0 = ( "); 
        sb.append("        SELECT COUNT(1) "); 
        sb.append("        FROM T172 "); 
        sb.append("        WHERE E161_PRJ_NBR = A.E161_PRJ_NBR "); 
        sb.append("        AND E100_PO_NBR = A.E100A_LCT_PO_NBR "); 
        sb.append("        AND ( "); 
        sb.append("           TOT_RTR_QTY > 0 "); 
        sb.append("           OR TOT_RFN_QTY > 0 "); 
        sb.append("           OR TOT_EXC_QTY > 0 "); 
        sb.append("        ) "); 
        sb.append("    ) "); 
        sb.append("ORDER BY C.P028_VND_NBR "); 
        sb.append("FOR READ ONLY ");
        
        return sb.toString();
    }
}